# Autogenerated by Astropy-affiliated package skypy's setup.py on 2020-07-14 19:42:05 UTC
from __future__ import unicode_literals
import datetime

version = "0.1.1rc1"
githash = "70a4649272c7fb48ed26f740ab127470bbea9ab5"


major = 0
minor = 1
bugfix = 1

version_info = (major, minor, bugfix)

release = True
timestamp = datetime.datetime(2020, 7, 14, 19, 42, 5)
debug = False

astropy_helpers_version = "3.2.1"
